package com.lee.device_helper;

import android.annotation.SuppressLint;
import android.app.ActivityManager;
import android.content.Context;
import android.location.LocationManager;
import android.net.ConnectivityManager;
import android.net.Network;
import android.net.NetworkCapabilities;
import android.net.wifi.WifiInfo;
import android.net.wifi.WifiManager;
import android.os.Build;
import android.os.Environment;
import android.os.StatFs;
import android.os.storage.StorageManager;
import android.provider.Settings;
import android.telephony.TelephonyManager;
import android.text.TextUtils;
import android.text.format.Formatter;
import androidx.annotation.NonNull;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;

import java.io.*;
import java.lang.reflect.Method;
import java.net.Inet4Address;
import java.net.InetAddress;
import java.net.NetworkInterface;
import java.net.SocketException;
import java.util.Enumeration;
import java.util.Locale;

/** DeviceHelperPlugin */
public class DeviceHelperPlugin implements FlutterPlugin, MethodCallHandler {
  /// The MethodChannel that will the communication between Flutter and native Android
  ///
  /// This local reference serves to register the plugin with the Flutter Engine and unregister it
  /// when the Flutter Engine is detached from the Activity
  private MethodChannel channel;
  private Context mContext;

  @Override
  public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
    channel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), "com.lee.device_helper/device_helper");
    channel.setMethodCallHandler(this);
    this.mContext = flutterPluginBinding.getApplicationContext();
  }

  @Override
  public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
    switch (call.method) {
      case "checkGPSIsOpen":
        result.success(checkGPSIsOpen());
        break;
      case "getAndroidId":
        result.success(getAndroidId());
        break;
      case "getAvailableInternalMemorySize":
        result.success(getAvailableInternalMemorySize());
        break;
      case "getTotalInternalMemorySize":
        result.success(getTotalInternalMemorySize());
        break;
      case "isSDCardMount":
        result.success(isSDCardMount());
        break;
      case "getSystemLanguage":
        result.success(getSystemLanguage());
        break;
      case "hasExtraSD":
        result.success(hasExtraSD());
        break;
      case "getIMEI":
        result.success(getIMEI());
        break;
      case "getDeviceId":
        result.success(getDeviceId());
        break;
      case "getIPAddress":
        result.success(getIPAddress());
        break;
      case "getMac":
        result.success(getMac());
        break;
      case "deviceBrand":
        result.success(getDeviceBrand());
        break;
      case "deviceType":
        result.success(getSystemModel());
        break;
      case "getAvailableRAM":
        result.success(getAvailableRAM());
        break;
      case "getTotalRAM":
        result.success(getTotalRAM());
        break;
      case "isRoot":
        result.success(isRoot());
        break;
      case "sdkVersion":
        result.success(sdkVersion());
        break;
      case "getSerial":
        result.success(getSerial());
        break;
      case "getSystemVersion":
        result.success(getSystemVersion());
        break;
      case "getSimOperator":
        result.success(getSimOperator());
        break;
      default:
        result.notImplemented();
        break;
    }
  }

  @Override
  public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
    channel.setMethodCallHandler(null);
    channel = null;
    mContext = null;
  }

  private boolean checkGPSIsOpen() {
    LocationManager manager = (LocationManager) mContext.getSystemService(Context.LOCATION_SERVICE);
    return manager.isProviderEnabled(LocationManager.GPS_PROVIDER);
  }

  @SuppressLint("HardwareIds")
  private String getAndroidId() {
    return Settings.Secure.getString(mContext.getContentResolver(), Settings.Secure.ANDROID_ID);
  }

  private String getAvailableInternalMemorySize() {
    File file = Environment.getDataDirectory();
    StatFs statFs = new StatFs(file.getPath());
    long availableBlocksLong = statFs.getAvailableBlocksLong();
    long blockSizeLong = statFs.getBlockSizeLong();
    return Formatter.formatFileSize(mContext, availableBlocksLong
            * blockSizeLong);
  }

  private String getTotalInternalMemorySize() {
    File file = Environment.getDataDirectory();
    StatFs statFs = new StatFs(file.getPath());
    long blockSizeLong = statFs.getBlockSizeLong();
    long blockCountLong = statFs.getBlockCountLong();
    long size = blockCountLong * blockSizeLong;
    return Formatter.formatFileSize(mContext, size);
  }

  private String getSystemLanguage() {
    return Locale.getDefault().getLanguage();
  }

  private boolean isSDCardMount() {
    return TextUtils.equals(Environment.getExternalStorageState(),
            Environment.MEDIA_MOUNTED);
  }

  private boolean hasExtraSD() {
    String path = getStoragePath(mContext, 1);
    return isSDCardMount() && path != null && !TextUtils.isEmpty(path);
  }

  /*
    内置SD卡: type 0
    外置SD卡: type 1
     */
  private String getStoragePath(Context context, int type) {
    StorageManager sm = (StorageManager) context.getSystemService(Context.STORAGE_SERVICE);
    try {
      Method getPathsMethod = sm.getClass().getMethod("getVolumePaths",
              String.class);
      String[] path = (String[])getPathsMethod.invoke(sm, Object.class.toString());

      switch (type){
        case 0:
          return path != null ? path[type] : null;
        case 1:
          if (path != null && path.length > 1) {
            return path[type];
          } else {
            return null;
          }

        default:
          break;
      }

    } catch (Exception e) {
      e.printStackTrace();
    }

    return null;
  }

  @SuppressLint("MissingPermission")
  private String getIMEI() {
    String imei;
    TelephonyManager mTelephonyMgr;
    try {
      mTelephonyMgr = (TelephonyManager) mContext.getSystemService(Context.TELEPHONY_SERVICE);
      if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
        imei = mTelephonyMgr.getImei();
      } else {
        imei = getDeviceId();
      }
    } catch (Exception e) {
      imei = getDeviceId();
    }

    if (null == imei) {
      imei = getDeviceId();
    }

    return imei;
  }

  @SuppressLint({"MissingPermission", "HardwareIds"})
  private String getDeviceId() {
    String deviceId;
    try {
      TelephonyManager tm = (TelephonyManager) mContext.getSystemService(Context.TELEPHONY_SERVICE);
      deviceId = tm.getDeviceId();//10.0之后获取不到
      if (TextUtils.isEmpty(deviceId)) {
        deviceId = Settings.Secure.getString(mContext.getContentResolver(),
                Settings.Secure.ANDROID_ID);
      }
    } catch (Exception e) {
      deviceId = Settings.Secure.getString(mContext.getContentResolver(),
              Settings.Secure.ANDROID_ID);
    }
    return deviceId;
  }

  @SuppressLint("MissingPermission")
  private String getIPAddress() {
    ConnectivityManager cm = (ConnectivityManager) mContext.getSystemService(Context.CONNECTIVITY_SERVICE);
    Network network = cm.getActiveNetwork();
    if (network != null && cm.getNetworkCapabilities(network) != null) {
      NetworkCapabilities capabilities = cm.getNetworkCapabilities(network);
      if (capabilities != null && capabilities.hasCapability(NetworkCapabilities.NET_CAPABILITY_INTERNET)) {
        if (capabilities.hasTransport(NetworkCapabilities.TRANSPORT_CELLULAR)) {
          try {
            for (Enumeration<NetworkInterface> en = NetworkInterface.getNetworkInterfaces(); en.hasMoreElements(); ) {
              NetworkInterface networkInterface = en.nextElement();
              for (Enumeration<InetAddress> enumIpAddress = networkInterface.getInetAddresses(); enumIpAddress.hasMoreElements(); ) {
                InetAddress inetAddress = enumIpAddress.nextElement();
                if (!inetAddress.isLoopbackAddress() && inetAddress instanceof Inet4Address) {
                  return inetAddress.getHostAddress();
                }
              }
            }
          } catch (SocketException e) {
            e.printStackTrace();
          }
        } else if (capabilities.hasTransport(NetworkCapabilities.TRANSPORT_WIFI)) {
          @SuppressLint("WifiManagerPotentialLeak") WifiManager wifiManager = (WifiManager) mContext.getSystemService(Context.WIFI_SERVICE);
          WifiInfo wifiInfo = wifiManager.getConnectionInfo();
          return intIP2StringIP(wifiInfo.getIpAddress());
        }
      }
    }

    return "";
  }

  private String intIP2StringIP(int ip) {
    return (ip & 0xFF) + "." +
            ((ip >> 8) & 0xFF) + "." +
            ((ip >> 16) & 0xFF) + "." +
            (ip >> 24 & 0xFF);
  }

  private String getMac() {
    String strMac = "02:00:00:00:00:00";
    if (Build.VERSION.SDK_INT < Build.VERSION_CODES.N && Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
      strMac = getMacAddress();
      return strMac;
    } else if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
      if (!TextUtils.isEmpty(getMacAddress())) {
        strMac = getMacAddress();
        return strMac;
      } else if (!TextUtils.isEmpty(getMachineHardwareAddress())) {
        strMac = getMachineHardwareAddress();
        return strMac;
      } else {
        strMac = getLocalMacAddressFromBusybox();
        return strMac;
      }
    }
    return strMac;
  }

  private String getMacAddress() {
    String str = "";
    String macSerial = "";
    try {
      Process pp = Runtime.getRuntime().exec(
              "cat /sys/class/net/wlan0/address");
      InputStreamReader ir = new InputStreamReader(pp.getInputStream());
      LineNumberReader input = new LineNumberReader(ir);
      while (null != str) {
        str = input.readLine();
        if (str != null) {
          macSerial = str.trim();// 去空格
          break;
        }
      }
    } catch (Exception ignored) {
    }

    if ("".equals(macSerial)) {
      try {
        return loadFileAsString()
                .toUpperCase().substring(0, 17);
      } catch (Exception e) {
        e.printStackTrace();
      }
    }

    return macSerial;
  }

  private String loadFileAsString() throws Exception {
    FileReader reader = new FileReader("/sys/class/net/eth0/address");
    String text = loadReaderAsString(reader);
    reader.close();
    return text;
  }

  private String loadReaderAsString(Reader reader) throws Exception {
    StringBuilder builder = new StringBuilder();
    char[] buffer = new char[4096];
    int readLength = reader.read(buffer);
    while (readLength >= 0) {
      builder.append(buffer, 0, readLength);
      readLength = reader.read(buffer);
    }

    return builder.toString();
  }

  private String getMachineHardwareAddress() {
    Enumeration<NetworkInterface> interfaces = null;
    try {
      interfaces = NetworkInterface.getNetworkInterfaces();
    } catch (SocketException e) {
      e.printStackTrace();
    }
    String hardWareAddress = "";
    NetworkInterface iF;
    if (interfaces == null) {
      return "";
    }

    while (interfaces.hasMoreElements()) {
      iF = interfaces.nextElement();
      try {
        hardWareAddress = bytesToString(iF.getHardwareAddress());
        break;
      } catch (SocketException e) {
        e.printStackTrace();
      }
    }
    return hardWareAddress;
  }

  private String bytesToString(byte[] bytes) {
    if (bytes == null || bytes.length == 0) {
      return "";
    }
    StringBuilder buf = new StringBuilder();
    for (byte b : bytes) {
      buf.append(String.format("%02X:", b));
    }
    if (buf.length() > 0) {
      buf.deleteCharAt(buf.length() - 1);
    }

    return buf.toString();
  }

  private String getLocalMacAddressFromBusybox() {
    String result;
    String Mac;
    result = callCmd();

    if (result.length() > 0 && result.contains("HWaddr")) {
      Mac = result.substring(result.indexOf("HWaddr") + 6,
              result.length() - 1);
      result = Mac;
    }

    return result;
  }

  private String callCmd() {
    StringBuilder result = new StringBuilder();
    String line;
    try {
      Process proc = Runtime.getRuntime().exec("busybox ifconfig");
      InputStreamReader is = new InputStreamReader(proc.getInputStream());
      BufferedReader br = new BufferedReader(is);

      while ((line = br.readLine()) != null && !line.contains("HWaddr")) {
        result.append(line);
      }

      result = new StringBuilder(line != null ? line : "");
    } catch (Exception e) {
      e.printStackTrace();
    }

    return result.toString();
  }

  private String getDeviceBrand() {
    return android.os.Build.BRAND;
  }

  public static String getSystemModel() {
    return android.os.Build.MODEL;
  }

  private String getAvailableRAM() {
    long size;
    ActivityManager activityManager = (ActivityManager) mContext.getSystemService(Context.ACTIVITY_SERVICE);
    ActivityManager.MemoryInfo outInfo = new ActivityManager.MemoryInfo();
    activityManager.getMemoryInfo(outInfo);
    size = outInfo.availMem;
    return Formatter.formatFileSize(mContext, size);
  }

  private String getTotalRAM() {
    long size;
    ActivityManager activityManager = (ActivityManager) mContext.getSystemService(Context.ACTIVITY_SERVICE);
    ActivityManager.MemoryInfo outInfo = new ActivityManager.MemoryInfo();
    activityManager.getMemoryInfo(outInfo);
    size = outInfo.totalMem;

    return Formatter.formatFileSize(mContext, size);
  }

  private boolean isRoot() {
    boolean root = false;
    try {
      root = (new File("/system/bin/su").exists()) || (new File("/system/xbin/su").exists());
    } catch (Exception ignored) {
    }

    return root;
  }

  private String sdkVersion() {
    return String.valueOf(Build.VERSION.SDK_INT);
  }

  @SuppressLint("HardwareIds")
  private String getSerial() {
    return Settings.Secure.getString(mContext.getContentResolver(), Settings.Secure.ANDROID_ID);
  }

  private String getSystemVersion() {
    return Build.VERSION.RELEASE;
  }

  private String getSimOperator() {
    TelephonyManager telephonyManager = (TelephonyManager) mContext.getSystemService(Context.TELEPHONY_SERVICE);
    return telephonyManager.getSimOperatorName();
  }
}

